@@ -2,20 +2,23 @@
 
 from __future__ import annotations
 
-import operator
 import warnings
-from typing import TYPE_CHECKING, Any
+from typing import TYPE_CHECKING, Any, Mapping
 
 import boost_histogram as bh
 import boost_histogram.axis as axis
 import boost_histogram.storage as storage
 import dask.array as da
-from dask.base import is_dask_collection
+from dask.base import DaskMethodsMixin, dont_optimize, is_dask_collection
+from dask.context import globalmethod
 from dask.delayed import Delayed, delayed
+from dask.highlevelgraph import HighLevelGraph
+from dask.threaded import get as tget
 from dask.utils import is_arraylike, is_dataframe_like
+from tlz import first
 
 from dask_histogram.bins import normalize_bins_range
-from dask_histogram.core import AggHistogram, factory
+from dask_histogram.core import AggHistogram, factory, optimize
 
 if TYPE_CHECKING:
     from dask_histogram.typing import (
@@ -31,7 +34,7 @@ import dask_histogram
 __all__ = ("Histogram", "histogram", "histogram2d", "histogramdd")
 
 
-class Histogram(bh.Histogram, family=dask_histogram):
+class Histogram(bh.Histogram, DaskMethodsMixin, family=dask_histogram):
     """Histogram object capable of lazy computation.
 
     Parameters
@@ -82,44 +85,68 @@ class Histogram(bh.Histogram, family=dask_histogram):
         """Construct a Histogram object."""
         super().__init__(*axes, storage=storage, metadata=metadata)
         self._staged: AggHistogram | None = None
+        self._dask_name = None
+        self._dask = None
 
-    def concrete_fill(
+    def __dask_graph__(self) -> HighLevelGraph:
+        return self._dask
+
+    def __dask_keys__(self) -> list[str]:
+        return [self.dask_name]
+
+    def __dask_layers__(self) -> tuple[str, ...]:
+        if isinstance(self._dask, HighLevelGraph) and len(self._dask.layers) == 1:
+            return tuple(self._dask.layers)
+        return (self.dask_name,)
+
+    def __dask_tokenize__(self) -> str:
+        return self.dask_name
+
+    def __dask_postcompute__(self) -> Any:
+        return lambda x: self._in_memory_type(first(x)), ()
+
+    def __dask_postpersist__(self) -> Any:
+        return self._rebuild, ()
+
+    __dask_optimize__ = globalmethod(
+        optimize, key="histogram_optimize", falsey=dont_optimize
+    )
+
+    __dask_scheduler__ = staticmethod(tget)
+
+    def _rebuild(
         self,
-        *args: Any,
-        weight: Any | None = None,
-        sample: Any | None = None,
-        threads: int | None = None,
-    ) -> Histogram:
-        """Fill the histogram with concrete data (not a Dask collection).
+        dsk: HighLevelGraph,
+        *,
+        rename: Mapping[str, str] | None = None,
+    ) -> Any:
+        dask_name = self._dask_name
+        if rename:
+            dask_name = rename.get(dask_name, dask_name)
+        new = type(self)(
+            *self.axes, storage=self.storage_type(), metadata=self.metadata
+        )
+        new._dask_name = dask_name
+        new._dask = dsk
+        return new
 
-        Calls the super class fill function
-        :py:func:`boost_histogram.Histogram.fill`.
-
-        Parameters
-        ----------
-        *args : array_like
-            Provide one value or array per dimension
-        weight : array_like, optional
-            Provide weights (only if the storage supports them)
-        sample : array_like
-            Provide samples (only if the storage supports them)
-        threads : int, optional
-            Fill with threads. Defaults to None, which does not
-            activate threaded filling. Using 0 will automatically pick
-            the number of available threads (usually two per core).
-
-        Returns
-        -------
-        dask_histogram.Histogram
-            Class instance now filled with concrete data.
-
-        """
-        if any(is_dask_collection(a) for a in args) or is_dask_collection(weight):
-            raise TypeError(
-                "concrete_fill does not support Dask collections, only materialized "
-                "data; use the Histogram.fill method."
+    @property
+    def _in_memory_type(self) -> type[bh.Histogram]:
+        if type(self) != Histogram:
+            warnings.warn(
+                """dask_histogram.boost.Histogram has been subclassed without
+                overriding '_in_memory_type', please do so if you would like to
+                receive an instance of subclass rather than a boost_histogram.Histogram!"""
             )
-        return super().fill(*args, weight=weight, sample=sample, threads=threads)
+        return bh.Histogram
+
+    @property
+    def dask_name(self) -> str:
+        return self._dask_name
+
+    @property
+    def dask(self) -> HighLevelGraph:
+        return self._dask
 
     def fill(  # type: ignore
         self,
@@ -181,14 +208,6 @@ class Histogram(bh.Histogram, family=dask_histogram):
             Class instance with a staged (delayed) fill added.
 
         """
-        # Pass to concrete fill if non-dask-collection
-        if all(not is_dask_collection(a) for a in args):
-            return self.concrete_fill(
-                *args,
-                weight=weight,
-                sample=sample,
-                threads=None,
-            )
 
         if len(args) == 1 and args[0].ndim == 1:
             pass
@@ -200,35 +219,14 @@ class Histogram(bh.Histogram, family=dask_histogram):
             raise ValueError(f"Cannot interpret input data: {args}")
 
         new_fill = factory(*args, histref=self, weights=weight, sample=sample)
-        if self._staged is not None:
-            self._staged += new_fill
-        else:
-            self._staged = new_fill
-
-        return self
-
-    def compute(self) -> Histogram:
-        """Compute any staged (delayed) fills.
-
-        Returns
-        -------
-        dask_histogram.Histogram
-            Concrete histogram with all staged (delayed) fills executed.
-
-        """
         if self._staged is None:
-            return self
-        if not self.empty():
-            result_view = self.view(flow=True) + self._staged.compute().view(flow=True)
+            self._staged = new_fill
         else:
-            result_view = self._staged.compute().view(flow=True)
-        self[...] = result_view
-        self._staged = None
-        return self
+            self._staged += new_fill
+        self._dask = self._staged.__dask_graph__()
+        self._dask_name = self._staged.name
 
-    def clear_fills(self) -> None:
-        """Drop any uncomputed fills."""
-        self._staged = None
+        return self
 
     def staged_fills(self) -> bool:
         """Check if histogram has staged fills.
@@ -283,9 +281,7 @@ class Histogram(bh.Histogram, family=dask_histogram):
         >>> h, = dask.compute(h.to_delayed())
 
         """
-        if self.staged_fills() and not self.empty():
-            return delayed(operator.add)(self._staged, delayed(self))
-        elif self._staged is not None:
+        if self._staged is not None:
             return self._staged.to_delayed()
         return delayed(bh.Histogram(self))
 
@@ -325,14 +321,6 @@ class Histogram(bh.Histogram, family=dask_histogram):
             ret += " # (has staged fills)"
         return ret
 
-    def visualize(self, *args: Any, **kwargs: Any) -> Any:
-        """Render the task graph with graphviz.
-
-        See :py:func:`dask.visualize` for supported keyword arguments.
-
-        """
-        return self.to_delayed().visualize(*args, **kwargs)
-
     def agg_histogram(self) -> AggHistogram | None:
         if self._staged is None:
             warnings.warn("No staged AggHistogram; returning None")
@@ -539,8 +527,6 @@ def histogramdd(
     >>> h.staged_fills()
     True
     >>> h = h.compute()
-    >>> h.staged_fills()
-    False
 
     """
 
